@@ -9,8 +9,9 @@ import hmac
 import struct
 import logging
 import sys
+import zlib
 from Pyro4 import errors, constants
-import Pyro4.constants
+from Pyro4.configuration import config
 
 
 __all__ = ["Message", "secure_compare"]
@@ -29,11 +30,6 @@ FLAGS_ONEWAY = 1 << 2
 FLAGS_BATCH = 1 << 3
 FLAGS_META_ON_CONNECT = 1 << 4
 FLAGS_ITEMSTREAMRESULT = 1 << 5
-SERIALIZER_SERPENT = 1
-SERIALIZER_JSON = 2
-SERIALIZER_MARSHAL = 3
-SERIALIZER_PICKLE = 4
-SERIALIZER_DILL = 5
 
 
 class Message(object):
@@ -101,8 +97,8 @@ class Message(object):
         if self.hmac_key:
             self.annotations["HMAC"] = self.hmac()   # should be done last because it calculates hmac over other annotations
         self.annotations_size = sum([6 + len(v) for v in self.annotations.values()])
-        if 0 < Pyro4.config.MAX_MESSAGE_SIZE < (self.data_size + self.annotations_size):
-            raise errors.MessageTooLargeError("max message size exceeded (%d where max=%d)" % (self.data_size + self.annotations_size, Pyro4.config.MAX_MESSAGE_SIZE))
+        if 0 < config.MAX_MESSAGE_SIZE < (self.data_size + self.annotations_size):
+            raise errors.MessageTooLargeError("max message size exceeded (%d where max=%d)" % (self.data_size + self.annotations_size, config.MAX_MESSAGE_SIZE))
 
     def __repr__(self):
         return "<%s.%s at %x; type=%d flags=%d seq=%d datasize=%d #ann=%d>" % (self.__module__, self.__class__.__name__, id(self), self.type, self.flags, self.seq, self.data_size, len(self.annotations))
@@ -166,8 +162,8 @@ class Message(object):
         """
         msg = cls.from_header(connection.recv(cls.header_size))
         msg.hmac_key = hmac_key
-        if 0 < Pyro4.config.MAX_MESSAGE_SIZE < (msg.data_size + msg.annotations_size):
-            errorMsg = "max message size exceeded (%d where max=%d)" % (msg.data_size + msg.annotations_size, Pyro4.config.MAX_MESSAGE_SIZE)
+        if 0 < config.MAX_MESSAGE_SIZE < (msg.data_size + msg.annotations_size):
+            errorMsg = "max message size exceeded (%d where max=%d)" % (msg.data_size + msg.annotations_size, config.MAX_MESSAGE_SIZE)
             log.error("connection " + str(connection) + ": " + errorMsg)
             connection.close()  # close the socket because at this point we can't return the correct sequence number for returning an error message
             exc = errors.MessageTooLargeError(errorMsg)
@@ -223,6 +219,14 @@ class Message(object):
         pyroConnection.send(ping.to_bytes())
         Message.recv(pyroConnection, [MSG_PING])
 
+    def decompress_if_needed(self):
+        """Decompress the message data if it is compressed."""
+        if self.flags & FLAGS_COMPRESSED:
+            self.data = zlib.decompress(self.data)
+            self.flags &= ~FLAGS_COMPRESSED
+            self.data_size = len(self.data)
+        return self
+
 
 try:
     from hmac import compare_digest as secure_compare
