@@ -4,19 +4,21 @@ Name Server and helper functions.
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
+from __future__ import print_function
 import warnings
-import random
 import re
 import logging
 import socket
 import sys
 import time
-from Pyro4.threadutil import RLock, Thread
+import threading
 from Pyro4.errors import NamingError, PyroError, ProtocolError
-from Pyro4 import core, socketutil
-import Pyro4.constants
+from Pyro4 import core, socketutil, constants
+from Pyro4.configuration import config
+from Pyro4.core import _locateNS as locateNS, _resolve as resolve    # API compatibility with older versions
 
-__all__ = ["locateNS", "resolve", "type_meta", "startNS", "startNSloop", "MemoryStorage"]
+
+__all__ = ["locateNS", "resolve", "type_meta", "startNS", "startNSloop", "startNS"]
 
 if sys.version_info >= (3, 0):
     basestring = str
@@ -62,7 +64,7 @@ class MemoryStorage(dict):
         pass
 
 
-@Pyro4.expose
+@core.expose
 class NameServer(object):
     """
     Pyro name server. Provides a simple flat name space to map logical object names to Pyro URIs.
@@ -73,7 +75,7 @@ class NameServer(object):
         if storageProvider is None:
             self.storage = MemoryStorage()
             log.debug("using volatile in-memory dict storage")
-        self.lock = RLock()
+        self.lock = threading.RLock()
 
     def count(self):
         return len(self.storage)
@@ -132,20 +134,20 @@ class NameServer(object):
 
     def remove(self, name=None, prefix=None, regex=None):
         """Remove a registration. returns the number of items removed."""
-        if name and name in self.storage and name != Pyro4.constants.NAMESERVER_NAME:
+        if name and name in self.storage and name != constants.NAMESERVER_NAME:
             with self.lock:
                 del self.storage[name]
             return 1
         if prefix:
             items = list(self.list(prefix=prefix).keys())
-            if Pyro4.constants.NAMESERVER_NAME in items:
-                items.remove(Pyro4.constants.NAMESERVER_NAME)
+            if constants.NAMESERVER_NAME in items:
+                items.remove(constants.NAMESERVER_NAME)
             self.storage.remove_items(items)
             return len(items)
         if regex:
             items = list(self.list(regex=regex).keys())
-            if Pyro4.constants.NAMESERVER_NAME in items:
-                items.remove(Pyro4.constants.NAMESERVER_NAME)
+            if constants.NAMESERVER_NAME in items:
+                items.remove(constants.NAMESERVER_NAME)
             self.storage.remove_items(items)
             return len(items)
         return 0
@@ -184,8 +186,7 @@ class NameServer(object):
                 result = {}
                 try:
                     regex = re.compile(regex)
-                except re.error:
-                    x = sys.exc_info()[1]
+                except re.error as x:
                     raise NamingError("invalid regex: " + str(x))
                 else:
                     for name in self.storage:
@@ -234,13 +235,13 @@ class NameServerDaemon(core.Daemon):
 
     def __init__(self, host=None, port=None, unixsocket=None, nathost=None, natport=None, storage=None):
         if host is None:
-            host = Pyro4.config.HOST
+            host = config.HOST
         if port is None:
-            port = Pyro4.config.NS_PORT
+            port = config.NS_PORT
         if nathost is None:
-            nathost = Pyro4.config.NATHOST
+            nathost = config.NATHOST
         if natport is None:
-            natport = Pyro4.config.NATPORT or None
+            natport = config.NATPORT or None
         storage = storage or "memory"
         if storage == "memory":
             log.debug("using volatile in-memory dict storage")
@@ -264,11 +265,11 @@ class NameServerDaemon(core.Daemon):
         if existing_count > 0:
             log.debug("number of existing entries in storage: %d", existing_count)
         super(NameServerDaemon, self).__init__(host, port, unixsocket, nathost=nathost, natport=natport)
-        self.register(self.nameserver, Pyro4.constants.NAMESERVER_NAME)
+        self.register(self.nameserver, constants.NAMESERVER_NAME)
         metadata = {"class:Pyro4.naming.NameServer"}
-        self.nameserver.register(Pyro4.constants.NAMESERVER_NAME, self.uriFor(self.nameserver), metadata=metadata)
-        if Pyro4.config.NS_AUTOCLEAN > 0:
-            if not AutoCleaner.override_autoclean_min and Pyro4.config.NS_AUTOCLEAN < AutoCleaner.min_autoclean_value:
+        self.nameserver.register(constants.NAMESERVER_NAME, self.uriFor(self.nameserver), metadata=metadata)
+        if config.NS_AUTOCLEAN > 0:
+            if not AutoCleaner.override_autoclean_min and config.NS_AUTOCLEAN < AutoCleaner.min_autoclean_value:
                 raise ValueError("NS_AUTOCLEAN cannot be smaller than " + str(AutoCleaner.min_autoclean_value))
             log.debug("autoclean enabled")
             self.cleaner_thread = AutoCleaner(self.nameserver)
@@ -314,7 +315,7 @@ class NameServerDaemon(core.Daemon):
             raise
 
 
-class AutoCleaner(Thread):
+class AutoCleaner(threading.Thread):
     """
     Takes care of checking every registration in the name server.
     If it cannot be contacted anymore, it will be removed after ~20 seconds.
@@ -325,8 +326,8 @@ class AutoCleaner(Thread):
     override_autoclean_min = False   # only for unit test purposes
 
     def __init__(self, nameserver):
-        assert Pyro4.config.NS_AUTOCLEAN > 0
-        if not self.override_autoclean_min and Pyro4.config.NS_AUTOCLEAN < self.min_autoclean_value:
+        assert config.NS_AUTOCLEAN > 0
+        if not self.override_autoclean_min and config.NS_AUTOCLEAN < self.min_autoclean_value:
             raise ValueError("NS_AUTOCLEAN cannot be smaller than "+str(self.min_autoclean_value))
         super(AutoCleaner, self).__init__()
         self.nameserver = nameserver
@@ -339,14 +340,14 @@ class AutoCleaner(Thread):
         while not self.stop:
             time.sleep(self.loop_delay)
             time_since_last_autoclean = time.time() - self.last_cleaned
-            if time_since_last_autoclean < Pyro4.config.NS_AUTOCLEAN:
+            if time_since_last_autoclean < config.NS_AUTOCLEAN:
                 continue
             for name, uri in self.nameserver.list().items():
-                if name in (Pyro4.constants.DAEMON_NAME, Pyro4.constants.NAMESERVER_NAME):
+                if name in (constants.DAEMON_NAME, constants.NAMESERVER_NAME):
                     continue
                 try:
-                    uri_obj = Pyro4.URI(uri)
-                    timeout = Pyro4.config.COMMTIMEOUT or 5
+                    uri_obj = core.URI(uri)
+                    timeout = config.COMMTIMEOUT or 5
                     sock = socketutil.createSocket(connect=(uri_obj.host, uri_obj.port), timeout=timeout)
                     sock.close()
                     # if we get here, the listed server is still answering on its port
@@ -366,8 +367,6 @@ class AutoCleaner(Thread):
 
 
 class BroadcastServer(object):
-    REQUEST_NSURI = b"GET_NSURI"
-
     class TransportServerAdapter(object):
         # this adapter is used to be able to pass the BroadcastServer to Daemon.combine() to integrate the event loops.
         def __init__(self, bcserver):
@@ -381,14 +380,14 @@ class BroadcastServer(object):
         self.transportServer = self.TransportServerAdapter(self)
         self.nsUri = nsUri
         if bcport is None:
-            bcport = Pyro4.config.NS_BCPORT
+            bcport = config.NS_BCPORT
         if bchost is None:
-            bchost = Pyro4.config.NS_BCHOST
+            bchost = config.NS_BCHOST
         if ":" in nsUri.host:  # ipv6
             bchost = bchost or "::"
-            self.sock = Pyro4.socketutil.createBroadcastSocket((bchost, bcport, 0, 0), reuseaddr=Pyro4.config.SOCK_REUSE, timeout=2.0)
+            self.sock = socketutil.createBroadcastSocket((bchost, bcport, 0, 0), reuseaddr=config.SOCK_REUSE, timeout=2.0)
         else:
-            self.sock = Pyro4.socketutil.createBroadcastSocket((bchost, bcport), reuseaddr=Pyro4.config.SOCK_REUSE, timeout=2.0)
+            self.sock = socketutil.createBroadcastSocket((bchost, bcport), reuseaddr=config.SOCK_REUSE, timeout=2.0)
         self._sockaddr = self.sock.getsockname()
         bchost = bchost or self._sockaddr[0]
         bcport = bcport or self._sockaddr[1]
@@ -416,7 +415,7 @@ class BroadcastServer(object):
 
     def runInThread(self):
         """Run the broadcast server loop in its own thread."""
-        thread = Thread(target=self.__requestLoop)
+        thread = threading.Thread(target=self.__requestLoop)
         thread.setDaemon(True)
         thread.start()
         log.debug("broadcast server loop running in own thread")
@@ -430,7 +429,7 @@ class BroadcastServer(object):
     def processRequest(self):
         try:
             data, addr = self.sock.recvfrom(100)
-            if data == self.REQUEST_NSURI:
+            if data == b"GET_NSURI":
                 responsedata = core.URI(self.nsUri)
                 if responsedata.host == "0.0.0.0":
                     # replace INADDR_ANY address by the interface IP address that connects to the requesting client
@@ -518,126 +517,6 @@ def startNS(host=None, port=None, enableBroadcast=True, bchost=None, bcport=None
     return nsUri, daemon, bcserver
 
 
-def locateNS(host=None, port=None, broadcast=True, hmac_key=None):
-    """Get a proxy for a name server somewhere in the network."""
-    if host is None:
-        # first try localhost if we have a good chance of finding it there
-        if Pyro4.config.NS_HOST in ("localhost", "::1") or Pyro4.config.NS_HOST.startswith("127."):
-            if ":" in Pyro4.config.NS_HOST:  # ipv6
-                hosts = ["[%s]" % Pyro4.config.NS_HOST]
-            else:
-                # Some systems (Debian Linux) have 127.0.1.1 in the hosts file assigned to the hostname,
-                # try this too for convenience sake (only if it's actually used as a valid ip address)
-                try:
-                    socket.gethostbyaddr("127.0.1.1")
-                    hosts = [Pyro4.config.NS_HOST] if Pyro4.config.NS_HOST == "127.0.1.1" else [Pyro4.config.NS_HOST, "127.0.1.1"]
-                except socket.error:
-                    hosts = [Pyro4.config.NS_HOST]
-            for host in hosts:
-                uristring = "PYRO:%s@%s:%d" % (Pyro4.constants.NAMESERVER_NAME, host, port or Pyro4.config.NS_PORT)
-                log.debug("locating the NS: %s", uristring)
-                proxy = core.Proxy(uristring)
-                proxy._pyroHmacKey = hmac_key
-                try:
-                    proxy._pyroBind()
-                    log.debug("located NS")
-                    return proxy
-                except PyroError:
-                    pass
-        if broadcast:
-            # broadcast lookup
-            if not port:
-                port = Pyro4.config.NS_BCPORT
-            log.debug("broadcast locate")
-            sock = Pyro4.socketutil.createBroadcastSocket(reuseaddr=Pyro4.config.SOCK_REUSE, timeout=0.7)
-            for _ in range(3):
-                try:
-                    for bcaddr in Pyro4.config.parseAddressesString(Pyro4.config.BROADCAST_ADDRS):
-                        try:
-                            sock.sendto(BroadcastServer.REQUEST_NSURI, 0, (bcaddr, port))
-                        except socket.error:
-                            x = sys.exc_info()[1]
-                            err = getattr(x, "errno", x.args[0])
-                            # handle some errno's that some platforms like to throw:
-                            if err not in Pyro4.socketutil.ERRNO_EADDRNOTAVAIL and err not in Pyro4.socketutil.ERRNO_EADDRINUSE:
-                                raise
-                    data, _ = sock.recvfrom(100)
-                    sock.close()
-                    if sys.version_info >= (3, 0):
-                        data = data.decode("iso-8859-1")
-                    log.debug("located NS: %s", data)
-                    proxy = core.Proxy(data)
-                    proxy._pyroHmacKey = hmac_key
-                    return proxy
-                except socket.timeout:
-                    continue
-            try:
-                sock.shutdown(socket.SHUT_RDWR)
-            except (OSError, socket.error):
-                pass
-            sock.close()
-            log.debug("broadcast locate failed, try direct connection on NS_HOST")
-        else:
-            log.debug("skipping broadcast lookup")
-        # broadcast failed or skipped, try PYRO directly on specific host
-        host = Pyro4.config.NS_HOST
-        port = Pyro4.config.NS_PORT
-    # pyro direct lookup
-    if not port:
-        port = Pyro4.config.NS_PORT
-    if core.URI.isUnixsockLocation(host):
-        uristring = "PYRO:%s@%s" % (Pyro4.constants.NAMESERVER_NAME, host)
-    else:
-        # if not a unix socket, check for ipv6
-        if ":" in host:
-            host = "[%s]" % host
-        uristring = "PYRO:%s@%s:%d" % (Pyro4.constants.NAMESERVER_NAME, host, port)
-    uri = core.URI(uristring)
-    log.debug("locating the NS: %s", uri)
-    proxy = core.Proxy(uri)
-    proxy._pyroHmacKey = hmac_key
-    try:
-        proxy._pyroBind()
-        log.debug("located NS")
-        return proxy
-    except PyroError as x:
-        e = NamingError("Failed to locate the nameserver")
-        if sys.version_info >= (3, 0):
-            e.__cause__ = x
-        raise e
-
-
-def resolve(uri, hmac_key=None):
-    """
-    Resolve a 'magic' uri (PYRONAME, PYROMETA) into the direct PYRO uri.
-    It finds a name server, and use that to resolve a PYRONAME uri into the direct PYRO uri pointing to the named object.
-    If uri is already a PYRO uri, it is returned unmodified.
-    You can consider this a shortcut function so that you don't have to locate and use a name server proxy yourself.
-    Note: if you need to resolve more than a few names, consider using the name server directly instead of repeatedly
-    calling this function, to avoid the name server lookup overhead from each call.
-    """
-    if isinstance(uri, basestring):
-        uri = core.URI(uri)
-    elif not isinstance(uri, core.URI):
-        raise TypeError("can only resolve Pyro URIs")
-    if uri.protocol == "PYRO":
-        return uri
-    log.debug("resolving %s", uri)
-    if uri.protocol == "PYRONAME":
-        with locateNS(uri.host, uri.port, hmac_key=hmac_key) as nameserver:
-            return nameserver.lookup(uri.object)
-    elif uri.protocol == "PYROMETA":
-        with locateNS(uri.host, uri.port, hmac_key=hmac_key) as nameserver:
-            candidates = nameserver.list(metadata_all=uri.object)
-            if candidates:
-                candidate = random.choice(list(candidates.values()))
-                log.debug("resolved to candidate %s", candidate)
-                return core.URI(candidate)
-            raise NamingError("no registrations available with desired metadata properties %s" % uri.object)
-    else:
-        raise PyroError("invalid uri protocol")
-
-
 def type_meta(class_or_object, prefix="class:"):
     """extracts type metadata from the given class or object, can be used as Name server metadata."""
     if hasattr(class_or_object, "__mro__"):
