@@ -10,8 +10,8 @@ import logging
 import linecache
 import traceback
 import inspect
-import Pyro4.errors
-import Pyro4.message
+from Pyro4 import errors
+from Pyro4.configuration import config
 
 try:
     import copyreg
@@ -48,7 +48,7 @@ def getPyroTraceback(ex_type=None, ex_value=None, ex_tb=None):
             ex_type, ex_value, ex_tb = sys.exc_info()
 
         remote_tb = getattr(ex_value, "_pyroTraceback", None)
-        local_tb = formatTraceback(ex_type, ex_value, ex_tb, Pyro4.config.DETAILED_TRACEBACK)
+        local_tb = formatTraceback(ex_type, ex_value, ex_tb, config.DETAILED_TRACEBACK)
         if remote_tb:
             remote_tb = formatRemoteTraceback(remote_tb)
             return local_tb + remote_tb
@@ -139,8 +139,8 @@ else:
     for name, t in vars(builtins).items():
         if type(t) is type and issubclass(t, BaseException):
             all_exceptions[name] = t
-for name, t in vars(Pyro4.errors).items():
-    if type(t) is type and issubclass(t, Pyro4.errors.PyroError):
+for name, t in vars(errors).items():
+    if type(t) is type and issubclass(t, errors.PyroError):
         all_exceptions[name] = t
 
 
@@ -194,13 +194,6 @@ class SerializerBase(object):
             return compressed, True
         return data, False
 
-    @classmethod
-    def decompress_if_needed(cls, message):
-        if message.flags & Pyro4.message.FLAGS_COMPRESSED:
-            message.data = zlib.decompress(message.data)
-            message.flags &= ~Pyro4.message.FLAGS_COMPRESSED
-        return message
-
     @classmethod
     def register_type_replacement(cls, object_type, replacement_function):
         raise NotImplementedError("implement in subclass")
@@ -220,7 +213,7 @@ class SerializerBase(object):
                     serializer.ser_builtins_dict(d, stream, level)
 
                 serpent.register_class(clazz, serpent_converter)
-            except Pyro4.errors.ProtocolError:
+            except errors.ProtocolError:
                 pass
 
     @classmethod
@@ -233,7 +226,7 @@ class SerializerBase(object):
             get_serializer_by_id(SerpentSerializer.serializer_id)
             import serpent
             serpent.unregister_class(clazz)
-        except Pyro4.errors.ProtocolError:
+        except errors.ProtocolError:
             pass
 
     @classmethod
@@ -299,7 +292,7 @@ class SerializerBase(object):
                 value["__class__"] = obj.__class__.__module__ + "." + obj.__class__.__name__
                 return value
             else:
-                raise Pyro4.errors.SerializeError("don't know how to serialize class " + str(obj.__class__) + " using serializer " + str(cls.__name__) + ". Give it vars() or an appropriate __getstate__")
+                raise errors.SerializeError("don't know how to serialize class " + str(obj.__class__) + " using serializer " + str(cls.__name__) + ". Give it vars() or an appropriate __getstate__")
 
     @classmethod
     def dict_to_class(cls, data):
@@ -315,19 +308,20 @@ class SerializerBase(object):
             converter = cls.__custom_dict_to_class_registry[classname]
             return converter(classname, data)
         if "__" in classname:
-            raise Pyro4.errors.SecurityError("refused to deserialize types with double underscores in their name: " + classname)
-        # because of efficiency reasons the constructors below are hardcoded here instead of added on a per-class basis to the dict-to-class registry
+            raise errors.SecurityError("refused to deserialize types with double underscores in their name: " + classname)
+        # for performance, the constructors below are hardcoded here instead of added on a per-class basis to the dict-to-class registry
         if classname.startswith("Pyro4.core."):
+            from Pyro4 import core  # XXX circular
             if classname == "Pyro4.core.URI":
-                uri = Pyro4.core.URI.__new__(Pyro4.core.URI)
+                uri = core.URI.__new__(core.URI)
                 uri.__setstate_from_dict__(data["state"])
                 return uri
             elif classname == "Pyro4.core.Proxy":
-                proxy = Pyro4.core.Proxy.__new__(Pyro4.core.Proxy)
+                proxy = core.Proxy.__new__(core.Proxy)
                 proxy.__setstate_from_dict__(data["state"])
                 return proxy
             elif classname == "Pyro4.core.Daemon":
-                daemon = Pyro4.core.Daemon.__new__(Pyro4.core.Daemon)
+                daemon = core.Daemon.__new__(core.Daemon)
                 daemon.__setstate_from_dict__(data["state"])
                 return daemon
         elif classname.startswith("Pyro4.util."):
@@ -342,12 +336,13 @@ class SerializerBase(object):
             elif classname == "Pyro4.util.SerpentSerializer":
                 return SerpentSerializer()
         elif classname.startswith("Pyro4.errors."):
-            errortype = getattr(Pyro4.errors, classname.split('.', 2)[2])
-            if issubclass(errortype, Pyro4.errors.PyroError):
+            errortype = getattr(errors, classname.split('.', 2)[2])
+            if issubclass(errortype, errors.PyroError):
                 return SerializerBase.make_exception(errortype, data)
         elif classname == "Pyro4.futures._ExceptionWrapper":
+            from Pyro4 import futures  # XXX circular
             ex = SerializerBase.dict_to_class(data["exception"])
-            return Pyro4.futures._ExceptionWrapper(ex)
+            return futures._ExceptionWrapper(ex)
         elif data.get("__exception__", False):
             if classname in all_exceptions:
                 return SerializerBase.make_exception(all_exceptions[classname], data)
@@ -375,7 +370,7 @@ class SerializerBase(object):
             if classname == serializer.__class__.__name__:
                 return serializer
         log.warning("unsupported serialized class: " + classname)
-        raise Pyro4.errors.SerializeError("unsupported serialized class: " + classname)
+        raise errors.SerializeError("unsupported serialized class: " + classname)
 
     @staticmethod
     def make_exception(exceptiontype, data):
@@ -418,13 +413,13 @@ class PickleSerializer(SerializerBase):
     A (de)serializer that wraps the Pickle serialization protocol.
     It can optionally compress the serialized data, and is thread safe.
     """
-    serializer_id = Pyro4.message.SERIALIZER_PICKLE
+    serializer_id = 4  # never change this
 
     def dumpsCall(self, obj, method, vargs, kwargs):
-        return pickle.dumps((obj, method, vargs, kwargs), Pyro4.config.PICKLE_PROTOCOL_VERSION)
+        return pickle.dumps((obj, method, vargs, kwargs), config.PICKLE_PROTOCOL_VERSION)
 
     def dumps(self, data):
-        return pickle.dumps(data, Pyro4.config.PICKLE_PROTOCOL_VERSION)
+        return pickle.dumps(data, config.PICKLE_PROTOCOL_VERSION)
 
     def loadsCall(self, data):
         return pickle.loads(data)
@@ -448,13 +443,13 @@ class DillSerializer(SerializerBase):
     A (de)serializer that wraps the Dill serialization protocol.
     It can optionally compress the serialized data, and is thread safe.
     """
-    serializer_id = Pyro4.message.SERIALIZER_DILL
+    serializer_id = 5  # never change this
 
     def dumpsCall(self, obj, method, vargs, kwargs):
-        return dill.dumps((obj, method, vargs, kwargs), Pyro4.config.DILL_PROTOCOL_VERSION)
+        return dill.dumps((obj, method, vargs, kwargs), config.DILL_PROTOCOL_VERSION)
 
     def dumps(self, data):
-        return dill.dumps(data, Pyro4.config.DILL_PROTOCOL_VERSION)
+        return dill.dumps(data, config.DILL_PROTOCOL_VERSION)
 
     def loadsCall(self, data):
         return dill.loads(data)
@@ -475,7 +470,7 @@ class DillSerializer(SerializerBase):
 
 class MarshalSerializer(SerializerBase):
     """(de)serializer that wraps the marshal serialization protocol."""
-    serializer_id = Pyro4.message.SERIALIZER_MARSHAL
+    serializer_id = 3  # never change this
 
     def dumpsCall(self, obj, method, vargs, kwargs):
         return marshal.dumps((obj, method, vargs, kwargs))
@@ -509,7 +504,7 @@ class MarshalSerializer(SerializerBase):
 
 class SerpentSerializer(SerializerBase):
     """(de)serializer that wraps the serpent serialization protocol."""
-    serializer_id = Pyro4.message.SERIALIZER_SERPENT
+    serializer_id = 1  # never change this
 
     def dumpsCall(self, obj, method, vargs, kwargs):
         return serpent.dumps((obj, method, vargs, kwargs), module_in_classname=True)
@@ -537,10 +532,16 @@ class SerpentSerializer(SerializerBase):
 
         serpent.register_class(object_type, custom_serializer)
 
+    @classmethod
+    def dict_to_class(cls, data):
+        if data.get("__class__") == "float":
+            return float(data["value"])     # serpent encodes a float nan as a special class dict like this
+        return super(SerpentSerializer, cls).dict_to_class(data)
+
 
 class JsonSerializer(SerializerBase):
     """(de)serializer that wraps the json serialization protocol."""
-    serializer_id = Pyro4.message.SERIALIZER_JSON
+    serializer_id = 2  # never change this
 
     __type_replacements = {}
 
@@ -586,21 +587,21 @@ def get_serializer(name):
     try:
         return _serializers[name]
     except KeyError:
-        raise Pyro4.errors.SerializeError("serializer '%s' is unknown or not available" % name)
+        raise errors.SerializeError("serializer '%s' is unknown or not available" % name)
 
 
 def get_serializer_by_id(sid):
     try:
         return _serializers_by_id[sid]
     except KeyError:
-        raise Pyro4.errors.SerializeError("no serializer available for id %d" % sid)
+        raise errors.SerializeError("no serializer available for id %d" % sid)
 
 # determine the serializers that are supported
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
-assert Pyro4.config.PICKLE_PROTOCOL_VERSION >= 2, "pickle protocol needs to be 2 or higher"
+assert config.PICKLE_PROTOCOL_VERSION >= 2, "pickle protocol needs to be 2 or higher"
 _ser = PickleSerializer()
 _serializers["pickle"] = _ser
 _serializers_by_id[_ser.serializer_id] = _ser
@@ -609,9 +610,6 @@ _ser = MarshalSerializer()
 _serializers["marshal"] = _ser
 _serializers_by_id[_ser.serializer_id] = _ser
 try:
-    import platform
-    if platform.python_implementation() in ('PyPy', 'IronPython'):
-        raise ImportError('Currently dill is not supported with PyPy and IronPython')
     import dill
     _ser = DillSerializer()
     _serializers["dill"] = _ser
@@ -621,9 +619,9 @@ except ImportError:
 try:
     try:
         import importlib
-        json = importlib.import_module(Pyro4.config.JSON_MODULE)
+        json = importlib.import_module(config.JSON_MODULE)
     except ImportError:
-        json = __import__(Pyro4.config.JSON_MODULE)
+        json = __import__(config.JSON_MODULE)
     _ser = JsonSerializer()
     _serializers["json"] = _ser
     _serializers_by_id[_ser.serializer_id] = _ser
@@ -637,7 +635,7 @@ try:
         ver = serpent.__version__
     ver = tuple(map(int, ver.split(".")))
     if ver < (1, 16):
-        raise RuntimeError("requires serpent 1.16 or better")
+        raise RuntimeError("requires serpent 1.17 or better")
     _ser = SerpentSerializer()
     _serializers["serpent"] = _ser
     _serializers_by_id[_ser.serializer_id] = _ser
@@ -658,7 +656,7 @@ def getAttribute(obj, attr):
         raise AttributeError("attempt to access private attribute '%s'" % attr)
     else:
         obj = getattr(obj, attr)
-    if not Pyro4.config.REQUIRE_EXPOSE or getattr(obj, "_pyroExposed", False):
+    if not config.REQUIRE_EXPOSE or getattr(obj, "_pyroExposed", False):
         return obj
     raise AttributeError("attempt to access unexposed attribute '%s'" % attr)
 
@@ -673,9 +671,8 @@ def fixIronPythonExceptionForPickle(exceptionObject, addAttributes):
     """
     Function to hack around a bug in IronPython where it doesn't pickle
     exception attributes. We piggyback them into the exception's args.
-    Bug report is at http://ironpython.codeplex.com/workitem/30805
-    migrated to github as https://github.com/IronLanguages/main/issues/943
-    Bug is still present in Ironpython 2.7.5
+    Bug report is at https://github.com/IronLanguages/main/issues/943
+    Bug is still present in Ironpython 2.7.7
     """
     if hasattr(exceptionObject, "args"):
         if addAttributes:
@@ -732,7 +729,7 @@ def get_exposed_members(obj, only_exposed=True, as_lists=False, use_cache=True):
         if inspect.ismethod(v) or inspect.isfunction(v):
             if getattr(v, "_pyroExposed", not only_exposed):
                 methods.add(m)
-                # check if the method is marked with the @Pyro4.oneway decorator:
+                # check if the method is marked with the 'oneway' decorator:
                 if getattr(v, "_pyroOneway", False):
                     oneway.add(m)
         elif inspect.isdatadescriptor(v):
